@@ -1,41 +1,32 @@
-import ast
+import os
 import platform
 from itertools import chain
 from pathlib import Path
 from typing import NamedTuple, Dict, Sequence, Tuple
 import importlib
 
-from connectome.storage import Storage, SSHLocation, Disk
-from connectome.storage.locker import RedisLocker
 from paramiko.config import SSHConfig
-from redis import Redis
 from yaml import safe_load
-import humanfriendly
 
-from .interface import Repository, PathLike
+from connectome.storage import Storage, SSHLocation, Disk
+from .utils import RepositoryNotFound, InconsistentRepositories, PathLike
+
+CONFIG = '.bev.yml'
 
 
+# TODO: pydantic
 class LocationMeta(NamedTuple):
     root: str
-    free: int = 0
-    size: int = None
     host: str = None
-    lock: str = None
-    lock_prefix_size: int = 16
-
-
-class CacheMeta(NamedTuple):
-    root: str
-    lock: str = None
 
 
 class StorageMeta(NamedTuple):
     locations: Sequence[LocationMeta]
-    cache: CacheMeta = None
+    cache: str = None
 
 
-def build_storage(path: Path) -> Tuple[Storage, CacheMeta]:
-    with open(path, 'r') as file:
+def build_storage(root: Path) -> Tuple[Storage, str]:
+    with open(root / CONFIG, 'r') as file:
         config = safe_load(file)
 
     entry, others = parse(config)
@@ -57,56 +48,45 @@ def build_storage(path: Path) -> Tuple[Storage, CacheMeta]:
                     'hostname': location.host} or location.host in config.get_hostnames())
             ]
 
-    local = []
-    for location in entry.locations:
-        local.append(Disk(
-            location.root, location.free, location.size, locker=make_locker(location.lock),
-            lock_prefix_size=location.lock_prefix_size
-        ))
-
+    local = [Disk(location.root) for location in entry.locations]
     return Storage(local, remote), entry.cache
 
 
-def make_locker(locker):
-    if locker is None:
-        return
-
-    locker = locker.strip()
-    assert locker.endswith(')')
-    idx = locker.index('(')
-    name, args = locker[:idx], locker[idx:]
-    # TODO: add real name detection
-    assert name == 'Redis'
-    url, prefix = ast.literal_eval(args)
-    return RedisLocker(Redis(url), prefix, 10 * 60)
-
-
 def parse(config) -> Tuple[StorageMeta, Dict[str, StorageMeta]]:
-    filter_func = choose_by_hostname
-    if '__filter__' in config:
-        path, attr = config.pop('__filter__').rsplit('.', 1)
+    filter_func = default_choose
+    meta = config.pop('meta', {})
+    assert set(meta) <= {'choose', 'default'}
+    if 'choose' in meta:
+        path, attr = meta.pop('choose').rsplit('.', 1)
         filter_func = getattr(importlib.import_module(path), attr)
+    default_storage = meta.get('default')
 
     result = {}
     for name, meta in config.items():
         default = meta.pop('default', {})
-        locations = []
+        assert set(meta) <= {'storage', 'cache'}
 
+        locations = []
         for location in meta['storage']:
             keys = default.copy()
             keys.update(location)
-            for k in ['size', 'free']:
-                if k in keys:
-                    keys[k] = humanfriendly.parse_size(keys[k])
-
             locations.append(LocationMeta(**keys))
 
-        cache = None
-        if 'cache' in meta:
-            cache = CacheMeta(**meta['cache'])
-        result[name] = StorageMeta(locations, cache)
+        result[name] = StorageMeta(locations, meta.get('cache'))
+
+    if default_storage is not None and default_storage not in result:
+        raise ValueError(f'The default storage ({default_storage}) is not present in the config')
+
+    if len(result) == 1:
+        entry, = result.values()
+        result = {}
+    else:
+        name = choose_local(result, filter_func) or default_storage
+        if name is None:
+            raise ValueError('No matching entry in config')
+
+        entry = result.pop(name)
 
-    entry = result.pop(choose_local(result, filter_func))
     return entry, result
 
 
@@ -115,22 +95,25 @@ def choose_local(names, func):
         if func(name):
             return name
 
-    raise ValueError('No matching entry in config')
 
+def default_choose(key):
+    env = os.environ.get('BEV__REPOSITORY')
+    if env:
+        return key == env
 
-def choose_by_hostname(key):
     return key == platform.node()
 
 
-CONFIG = '.bev.yml'
-
-
-def get_current_repo(path: PathLike = '.') -> Repository:
+def _find_root(path, marker):
     path = Path(path).resolve()
     for parent in chain([path], path.parents):
-        config = parent / CONFIG
-        if config.exists():
-            storage, cache = build_storage(config)
-            return Repository(parent, storage, cache)
+        if (parent / marker).exists():
+            return parent
 
-    raise FileNotFoundError(f'{CONFIG} files not found in current folder\'s parents')
+
+def find_repo_root(path: PathLike):
+    return _find_root(path, CONFIG)
+
+
+def find_vcs_root(path: PathLike):
+    return _find_root(path, '.git')
